/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_APPLICATION_H
#define SMART_TOOLS_APPLICATION_H

#include "stdir.h"
#include "stfile.h"
#include "stthread.h"
#include "smarttools.h"
#include "ststringlist.h"

#include <sys/types.h>

const int ER_STAPPLICATION_TEMP_FILE = 1;

/*!
	\brief StApplication is representation of application with some arguments.
	
	Let's think of StApplication like single line in bash shell. This can be for
	example "ls" and for this task we use setExecutable(). We can attach to it some
	arguments like "ls -alh" by setArguments(). When we execute() application like
	that, it will generate output and error output accessable by getNormalOutput() and
	getErrorOutput() and it will also return exit code in getReturnCode();
	
	\warning Don't forget to setTempDirectory() before execute(). If you forget that,
		temporary files will be saved directly in /tmp directory which is dangerous
		for some reasons.
*/

class StApplication : public StThread
{
	public:
		StApplication(){ init(); clear(); }
		StApplication( const StFile& file, const StDir& dir=StDir(ST_DEFAULT_TEMP) )
			{ init(); setExecutable( file ); setTempDirectory( dir ); }
		StApplication( const StFile& file, const StStringList& args, const StDir& dir=StDir(ST_DEFAULT_TEMP) )
			{ init(); setExecutable( file ); setArguments( args ), setTempDirectory( dir ); }
		
		/** 
			Set where is binary to execute.
			\return true if binary is executable.
		*/
		bool setExecutable( const StFile& file );
		
		/** Where program will be execute. */
		void setWorkDirectory( const StDir& dir ) { workDir = dir; }
		
		/** Where output files will be stored while executing. */
		void setTempDirectory( const StDir& dir );
		
		/** With what arguments execute binary. */
		void setArguments( const StStringList& args ){ this->args = args; }
		
		/** Overloaded version of function above. */
		void setArguments( const StString& arg ){ setArguments( StStringList() << arg ); }
		
		/*!
			Executes application
			\param wait wait for application to close
			\param outputToFile if true, application output will be saved to file without displaying it
			\return true if execute is done
		*/
		bool execute( bool wait=true, bool outputToFile=true );
		
		/** \return Output from stdout */
		StStringList getNormalOutput(){ return normalOutput; }
		/** \return Output from stderr */
		StStringList getErrorOutput(){ return errorOutput; }
		
		/** \return Application exit code */
		int getReturnCode(){ return WEXITSTATUS(returnCode); }
	
	protected:
		StDir tempDir;
		StDir workDir;
		
		virtual void run();
	
	private:
		StFile executable;
		
		StStringList args;
		
		volatile int returnCode;
		volatile bool outputToFile;
		volatile pid_t childPid;
		
		StStringList normalOutput, errorOutput; 
		
		void init(){ returnCode = 0; tempDir=ST_DEFAULT_TEMP; workDir="."; }
		void clear(){ setTempDirectory(ST_DEFAULT_TEMP); setExecutable(""); }
		
		StFile genFileName() const;
		
};

#endif //SMART_TOOLS_APPLICATION_H
